@@ -1,11 +1,30 @@
+#    Copyright (C) 2016 University of Southern California and
+#             Chao Deng and Andrew D. Smith and Timothy Daley
+#
+#    Authors: Chao Deng
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+
 ### initial settings of two parameters size and mu in a negative binomial
 ### distribution for a numeric optimal searching function optim in R
-SIZE.INIT = 1
-MU.INIT = 0.5
+SIZE.INIT <- 1
+MU.INIT <- 0.5
 
 ### termination conditions for EM algorithm
-TOLERANCE = 1e-10
-ITER.TOLERANCE = 1e5
+TOLERANCE <- 1e-10
+ITER.TOLERANCE <- 1e5
 
 
 ### density function of a truncated zero negative binomial distribution
@@ -34,31 +53,31 @@ zerotruncated.dnbinom <- function(x, size, mu, log = FALSE)
 }
 
 
-### zerotruncated negative loglikelyhood 
-zerotruncated.minus.log.likelyhood <- function(hist.table, size, mu)
+### zerotruncated negative loglikelihood
+zerotruncated.minus.log.likelihood <- function(hist.table, size, mu)
 {
   prob <- zerotruncated.dnbinom(hist.table[, 1], size, mu, log = TRUE)
 
-  ## negative loglikelyhood
+  ## negative loglikelihood
   prob <- -prob
   return( prob %*% hist.table[, 2] )
 }
 
 
-### calculate the negative binomial loglikelyhood
+### calculate the negative binomial loglikelihood
 ### zero.items is number of items unobserved
 ### size and mu are parameters in a negative binomial distribution
-nb.loglikelyhood <- function(hist.table, zero.items, size, mu)
+nb.loglikelihood <- function(hist.table, zero.items, size, mu)
 {
-  ## likelyhood of nonzero terms
+  ## likelihood of nonzero terms
   log.prob <- dnbinom(hist.table[, 1], size = size, mu = mu, log = TRUE)
-  loglikelyhood <- log.prob %*% hist.table[, 2]
+  loglikelihood <- log.prob %*% hist.table[, 2]
 
   ## add items with zero count
   log.zero.prob <- dnbinom(0, size = size, mu = mu, log = TRUE)
-  loglikelyhood <- loglikelyhood + zero.items * log.zero.prob
+  loglikelihood <- loglikelihood + zero.items * log.zero.prob
 
-  return(loglikelyhood)
+  return(loglikelihood)
 }
 
 
@@ -69,6 +88,7 @@ preseqR.ztnb.em <- function(n, size=SIZE.INIT, mu=MU.INIT)
 {
   checking.hist(n)
 
+  n[, 2] <- as.numeric(n[, 2])
   ## setting the number of unobserved items as 0
   zero.prob <- exp(dnbinom(0, size = size, mu = mu, log = TRUE))
 
@@ -79,27 +99,24 @@ preseqR.ztnb.em <- function(n, size=SIZE.INIT, mu=MU.INIT)
   ## expected the number of unobservations
   zero.items <- L*zero.prob
 
-  ## convert zero.items into an integer
-  zero.items <- floor(zero.items)
-
   ## estimated mean and variance
   m <- (n[, 1] %*% n[, 2]) / L
   v <- ( (n[, 1] - m)^2 %*% n[, 2] + m^2 * zero.items )/(L - 1)
 
   ## target function f
   f <- function(x) {
-        return( -nb.loglikelyhood(n, zero.items, size = x, mu = m)/L )
+        return( -nb.loglikelihood(n, zero.items, size = x, mu = m)/L )
   }
 
   ## derivative of f
-  gr <- function(x) 
+  gr <- function(x)
   {
-    first.term <- ( digamma(x) * zero.items + 
+    first.term <- ( digamma(x) * zero.items +
                     digamma(n[, 1] + size) %*% n[, 2] )/L
     second.term <- digamma(x)
     third.term <- log(x) - log(x + m)
     result <- first.term - second.term + third.term
-    # f is negative loglikelyhood
+    # f is negative loglikelihood
     return(-result)
   }
 
@@ -115,18 +132,18 @@ preseqR.ztnb.em <- function(n, size=SIZE.INIT, mu=MU.INIT)
   ## count the times of iteration
   iter <- as.double(1)
 
-  ## initialize the negative loglikelyhood
-  loglikelyhood.pre <- Inf
+  ## initialize the negative loglikelihood
+  loglikelihood.pre <- Inf
 
-  ## zerotruncated loglikelyhood 
-  loglikelyhood <- zerotruncated.minus.log.likelyhood(n, res$par, m)
+  ## zerotruncated loglikelihood
+  loglikelihood <- zerotruncated.minus.log.likelihood(n, res$par, m)
 
   ## EM algorithm
-  while (( loglikelyhood.pre - loglikelyhood )/observed.items > TOLERANCE &&
+  while (( loglikelihood.pre - loglikelihood )/observed.items > TOLERANCE &&
            iter < ITER.TOLERANCE)
   {
-    ## update negative loglikelyhood
-    loglikelyhood.pre <- loglikelyhood
+    ## update negative loglikelihood
+    loglikelihood.pre <- loglikelihood
 
     ## update parameters
     size <- res$par
@@ -143,9 +160,6 @@ preseqR.ztnb.em <- function(n, size=SIZE.INIT, mu=MU.INIT)
     ## update expected number of unobserved items
     zero.items <- L*zero.prob
 
-    ## convert zero.items into an integer
-    zero.items <- floor(zero.items)
-
     ## estimated mean and variance
     m <- (n[, 1] %*% n[, 2])/L
     v <- ( (n[, 1] - m)^2 %*% n[, 2] + m^2 * zero.items )/(L - 1)
@@ -159,10 +173,10 @@ preseqR.ztnb.em <- function(n, size=SIZE.INIT, mu=MU.INIT)
              lower = 0.0001, upper = 10000)
     }
     iter <- iter + 1
-    ## zerotruncated loglikelyhood
-    loglikelyhood <- zerotruncated.minus.log.likelyhood(n, res$par, m)
+    ## zerotruncated loglikelihood
+    loglikelihood <- zerotruncated.minus.log.likelihood(n, res$par, m)
   }
-  return(list(size = size, mu = mu, loglik = -loglikelyhood.pre))
+  return(list(size = size, mu = mu, loglik = -loglikelihood.pre))
 }
 
 
@@ -172,6 +186,7 @@ preseqR.ztnb.estimate <- function(n, t, size=SIZE.INIT, mu=MU.INIT)
 {
   checking.hist(n)
 
+  n[, 2] <- as.numeric(n[, 2])
   distinct <- sum(n[, 2])
 
   ## estimate the parameters
@@ -203,7 +218,9 @@ preseqR.ztnb.species.accum.curve <- function(n, ss = NULL, max.extrapolation = N
 {
   checking.hist(n)
 
-  total.sample <- floor(n[, 1] %*% n[, 2])
+  n[, 2] <- as.numeric(n[, 2])
+
+  total.sample <- n[, 1] %*% n[, 2]
   distinct <- sum(n[, 2])
 
   ## set step.size = total.sample if it is undefined
@@ -219,14 +236,14 @@ preseqR.ztnb.species.accum.curve <- function(n, ss = NULL, max.extrapolation = N
 
     ## T is the number of experiments; 100 is a magic number
     max.extrapolation <- 100*total.sample
-    T <- as.integer( max.extrapolation/ss )
+    T <- floor( max.extrapolation/ss )
 
   } else if (max.extrapolation < ss) {
     write("max.extrapolation should be no less then ss", stderr())
     return(NULL)
   } else {
     # T is the number of experiments
-    T <- as.integer( max.extrapolation/ss )
+    T <- floor( max.extrapolation/ss )
   }
 
   sample.size <- as.double(ss) * (1: T)
@@ -243,14 +260,14 @@ preseqR.ztnb.species.accum.curve <- function(n, ss = NULL, max.extrapolation = N
   L <- distinct/p
 
   ## estimate the item being sampled under new experiments with different size
-  t = sample.size/total.sample
-  dim(t) = length(t)
-  P = apply(t, 1, function(x) {1 - dnbinom(0, size, mu = x * mu)})
-  yield.estimates = L*P
+  t <- sample.size/total.sample
+  dim(t) <- length(t)
+  P <- apply(t, 1, function(x) {1 - dnbinom(0, size, mu = x * mu)})
+  yield.estimates <- L*P
 
   ## combine sample.size and yield.estimates into matrix
-  yield.estimates = matrix(c(sample.size, yield.estimates), ncol = 2, byrow = FALSE)
-  colnames(yield.estimates) = c('sample.size', 'yield.estimate')
+  yield.estimates <- matrix(c(sample.size, yield.estimates), ncol = 2, byrow = FALSE)
+  colnames(yield.estimates) <- c('sample.size', 'yield.estimate')
   return(yield.estimates)
 }
 
@@ -264,7 +281,8 @@ preseqR.ztnb.mincount <- function(n, ss = NULL, max.extrapolation = NULL, r=1,
 {
   checking.hist(n)
 
-  total.sample <- floor(n[, 1] %*% n[, 2])
+  n[, 2] <- as.numeric(n[, 2])
+  total.sample <- n[, 1] %*% n[, 2]
   distinct <- sum(n[, 2])
 
   ## set step.size = total.sample if it is undefined
@@ -280,14 +298,14 @@ preseqR.ztnb.mincount <- function(n, ss = NULL, max.extrapolation = NULL, r=1,
 
     ## T is the number of experiments; 100 is a magic number
     max.extrapolation <- 100*total.sample
-    T <- as.integer( max.extrapolation/ss )
+    T <- floor( max.extrapolation/ss )
 
   } else if (max.extrapolation < ss) {
     write("max.extrapolation should be no less then ss", stderr())
     return(NULL)
   } else {
     # T is the number of experiments
-    T <- as.integer( max.extrapolation/ss )
+    T <- floor( max.extrapolation/ss )
   }
 
   sample.size <- as.double(ss) * (1: T)
@@ -304,15 +322,15 @@ preseqR.ztnb.mincount <- function(n, ss = NULL, max.extrapolation = NULL, r=1,
   L <- distinct/p
 
   ## estimate the item being sampled under new experiments with different size
-  t = sample.size/total.sample
-  dim(t) = length(t)
-  max.r = max(r)
-  yield.estimates = list()
-  P = rep(1, length(t))
+  t <- sample.size/total.sample
+  dim(t) <- length(t)
+  max.r <- max(r)
+  yield.estimates <- list()
+  P <- rep(1, length(t))
   for (i in 1:max.r) {
-    P = P - apply(t, 1, function(x) dnbinom(i-1, size, mu=x*mu))
+    P <- P - apply(t, 1, function(x) dnbinom(i-1, size, mu=x*mu))
     if (i %in% r) {
-      yield.estimates = c(yield.estimates, list(L * P))
+      yield.estimates <- c(yield.estimates, list(L * P))
     }
   }
 
@@ -320,7 +338,7 @@ preseqR.ztnb.mincount <- function(n, ss = NULL, max.extrapolation = NULL, r=1,
   #      apply(t, 1, function(y) dnbinom(x, size, mu = y * mu))
   #      })
   #yield.estimates = lapply(1:length(r), function(x) {L * P[[x]]})
-  yield.estimates = lapply(1:length(r), function(x) {
+  yield.estimates <- lapply(1:length(r), function(x) {
       estimates <- matrix(c(sample.size, yield.estimates[[x]]), ncol = 2, byrow = FALSE)
       colnames(estimates) <- c("sample.size", paste("yield.estimates(r=", r[x], ")", sep=""))
       estimates
