@@ -20,11 +20,12 @@
 #include "continued_fraction.h"
 #include <vector>
 #include <cmath>
-#include <R.h>
 
+#include <R_ext/Arith.h> // for R_FINITE
 
 using std::vector;
 using std::min;
+using std::fabs;
 
 const double TOLERANCE = 1e-20;
 
@@ -434,7 +435,7 @@ check_yield_estimates_stability(const vector<double> &estimates) {
   // make sure that the estimate is increasing in the time_step and
   // is below the initial distinct per step_size
   for (size_t i = 0; i < estimates.size(); ++i)
-	  if (!(R_FINITE(estimates[i])) || estimates[i] < 0)
+	  if (!R_FINITE(estimates[i]) || estimates[i] < 0)
 		  return false;
   for (size_t i = 1; i < estimates.size(); ++i){
     if (estimates[i] < estimates[i - 1] ){
@@ -512,48 +513,3 @@ ContinuedFractionApproximation::optimal_cont_frac_distinct(const vector<double>
    // no stable continued fraction: return null
   return ContinuedFraction();  
 }
-
-// find the optimal RFA for a given power series with coefficients ps_coeff
-ContinuedFraction
-ContinuedFractionApproximation::optimal_powerseries_to_cont_frac(
-                                  const std::vector<double> &ps_coeffs) const {
-  if (max_terms > ps_coeffs.size()) {
-	  ContinuedFraction empty;
-	  return empty;
-  }
-  
-  ContinuedFraction full_CF(ps_coeffs, diagonal_idx, max_terms);  
-  // if max terms = 4, check only that degree
-  if(max_terms == 4 || max_terms == 3 
-     || max_terms == 5 || max_terms == 6){   
-    vector<double> estimates;
-    full_CF.extrapolate_distinct(SEARCH_MAX_VAL, SEARCH_STEP_SIZE, estimates);
-    // return the continued fraction if it is stable
-    if (check_yield_estimates_stability(estimates))
-      return full_CF;
-  }
-  else{
-    //if max terms >= 8, start at 8 and check increasing cont frac's
-    //may change the starting terms for k > 1
-    size_t curr_terms = 0;
-    if(max_terms % 2 == 0)
-      curr_terms = 8;
-    else
-      curr_terms = 7;
-    while (curr_terms <= max_terms) {    
-      ContinuedFraction curr_cf 
-	= ContinuedFraction::truncate_degree(full_CF, curr_terms);
-      vector<double> estimates;
-      curr_cf.extrapolate_distinct(SEARCH_MAX_VAL, SEARCH_STEP_SIZE, estimates);
-          
-    // return the continued fraction if it is stable
-      if (check_yield_estimates_stability(estimates))
-	      return curr_cf;
-    
-      curr_terms += 2;
-    // if not cf not acceptable, increase degree
-    }
-  }
-   // no stable continued fraction: return null
-  return ContinuedFraction();  
-}
